'##############################################################################
' RelaisUndLicht.BAS:                                          Stand 11.12.2007
' -------------------                                          (C) Markus Fulde
' Testprogramm zur Inbetriebnahme der Relais fr Beschattung und
' Dimmung.
' Die Inbetriebnahme erfolgt mit dem Eva-Board STK500 + STK501
' Die fr Ansteuerung von Relais und Dimmer notwendigen Routinen
' sind als solche gekennzeichnet und bereits zur spteren Verwendung
' ausgelagert.
'##############################################################################


'------------------------------------------------------------------------------
' Compilerinstruktionen und Compilerdirektiven
'------------------------------------------------------------------------------
$regfile = "m128def.dat"
$crystal = 16000000
$hwstack = 32
$swstack = 32
$framesize = 40

$baud = 38400

'------------------------------------------------------------------------------
' Definition von Ressourcen
'------------------------------------------------------------------------------

'========== Fr Projekt ConvCtrl ==========

' REL: Alias fr Relais-Steuerung
Out_auf Alias Pinc.6                                        ' fr Relais Beschattung auf
Out_zu Alias Pinc.7                                         ' fr Relais Beschattung zu
Out_licht Alias Ping.0                                      ' fr Relais Licht an

Rel_auf_on Alias Portc.6
Rel_zu_on Alias Portc.7
Rel_licht_on Alias Portg.0

' DIM: Alias fr Dimmer_Steuerung
Null_detect Alias Pine.5                                    ' fr Input-PIN
Null_detect_pin Alias Porte.5                               ' fr PullUp

Ignition Alias Pind.4                                       ' fr Trias-Zndung
Triac_start Alias Portd.4


'========== Fr Testumgebung  ==========

' Alias Namen fr Tastaturabfragen
Taste_1 Alias Pina.0
Taste_2 Alias Pina.1
Taste_3 Alias Pina.2
Taste_4 Alias Pina.3
Taste_5 Alias Pina.4
Taste_6 Alias Pina.5
Taste_7 Alias Pina.6
Taste_8 Alias Pina.7

' Alias Namen fr LED's
Led0 Alias Portb.0
Led1 Alias Portb.1
Led2 Alias Portb.2
Led3 Alias Portb.3
Led4 Alias Portb.4
Led5 Alias Portb.5
Led6 Alias Portb.6
Led7 Alias Portb.7



'------------------------------------------------------------------------------
' Definition von Variablen, Konstaten und Datentypen
'------------------------------------------------------------------------------

'========== Fr Projekt ConvCtrl ==========

' REL: Konstanen
Const Passiv = 0
Const Aktiv = 255
Const Beschattung_delay = 10
Const Relais_testmodus = 0

' DIM: Konstanten
Const Dimmer_testmodus = 0
Const Dim_helligkeit_default = 16
Const Dim_helligkeitstabellenwerte = 32
Const Dim_helligkeit_min = 3
Const Dim_helligkeit_max = 30

' REL: Variablen definieren
Dim Beschattung_auf_flag As Byte
Dim Beschattung_zu_flag As Byte
Dim Beschattung_auf_counter As Word
Dim Beschattung_zu_counter As Word

' DIM: Variablen definieren
Dim Sinuscounter As Word
Sinuscounter = 0

Dim Triac_zuenszeitpunkte(32) As Byte
Dim Dim_helligkeitsstufe As Byte
Dim Dim_index_counter As Byte


'========== Fr Testumgebung ==========

' Konstanten
Const Sequencer_testmodus = 0
Const Timer_testmodus = 0
Const Main_testmodus = 0
Const Timervorgabe = 3036                                   ' Timer von 1 Sekunden

Const Led_aus = 1
Const Led_an = 0

' Sonstige Variablen definieren
Dim Timercounter As Word
Timercounter = 0

Dim Beleuchtung_flag As Byte
Beleuchtung_flag = Passiv

Dim Free_running_flag As Byte
Free_running_flag = Passiv


'------------------------------------------------------------------------------
' Prototyping
'------------------------------------------------------------------------------

'========== Fr Projekt ConvCtrl ==========

'========== Fr Testumgebung ==========


'------------------------------------------------------------------------------
' Konfiguration und Basiseinstellungen (Projekt und Testumgebung)
'------------------------------------------------------------------------------

'========== Fr Testumgebung ==========

'Konfiguration des Port A als Eingang fr Tastaturabfrage
Config Porta = Input

'Konfiguration des Port B als Ausgang fr Statusanzeige via LED
Config Portb = Output
Portb = &B11111111

'Konfiguration eines Timers fr 1 Sekunden Timer-Tick
Config Timer1 = Timer , Prescale = 256                      ' Timer 1 verwenden
On Timer1 Timer_irq                                         ' Interrupt Routine
Timer1 = Timervorgabe


'========== Fr Projekt ConvCtrl ==========

'-- Initialisierung Relais --

' REL: Routinen und Einstellungen
Gosub Relais_konfig

'Hier werden die Timer aktiviert
Enable Timer1                                               ' Timer 1 freigeben

'-- Initialisierung Dimmer --

' DIM: Interrupt fr Nulldurchgangserkennung konfigurieren
Gosub Dimmer_konfig

' INT5 so konfiguriert das Trigger bei steigender  Flanke
Config Int5 = Rising
' Springe zur Interrupt-Routine Dimmer_irq
On Int5 Dimmer_null_irq
Enable Int5                                                 ' Interrupt5 freigeben


'Konfiguration Timer2 fr Zndung TRIAC nach Vorgabe bzw. Nulldurchgang
Config Timer2 = Timer , Prescale = 1024
On Timer2 Dimmer_triac_irq
Enable Timer2


'-- Interrupts global freigeben --
Enable Interrupts

'-- Timer0 sofort anhalten und erst ber SW anstossen
Stop Timer2

'========== Fr Testumgebung ==========

#if Main_testmodus
   Print "** Enter MainLoop **"
   Print
#endif


'##############################################################################
' Hauptprogramm
'##############################################################################
'Hier ist die Programmhauptschleife
Do
   ' ===== Tasten fr Relaissteuerung =====

   ' Tastaturabfrage Taste Beschattung ganz auf
   Debounce Taste_1 , 0 , Relais_beschattung_auf , Sub

   ' Tastaturabfrage Taste Beschattung ganz auf
   Debounce Taste_2 , 0 , Relais_beschattung_zu , Sub

   ' Tastaturabfrage Taste Beschattung stopp
   Debounce Taste_3 , 0 , Relais_beschattung_stop , Sub

   'Tastaturabfrage fr Toggle-Funktion Licht EIN/AUS
   Debounce Taste_4 , 0 , Beleuchtung_ein_aus , Sub


   ' ===== Tasten fr Dimmersteuerung =====
   ' Tastaturabfrage Taste Licht heller
   Debounce Taste_5 , 0 , Dimmer_heller , Sub

   ' Tastaturabfrage Taste Licht dunkler
   Debounce Taste_6 , 0 , Dimmer_dunkler , Sub

   ' Tastaturabfrage fr Toglle-Funktion Dimmer EIN/AUS
   Debounce Taste_7 , 0 , Dimmer_free_running , Sub

   ' Taste fr Triac ONE-Shot
   Debounce Taste_8 , 0 , Dimmer_one_shot , Sub

   ' ===== SW-Timer-Funktionen =====
   Gosub Sw_timer_sequencer

Loop
'## End Hauptprogramm #########################################################


'******************************************************************************
' Interruptroutinen
'******************************************************************************
'------------------------------------------------------------------------------
' Interrupt-Service-Routine (Timer1): Timer_irq
' Routine zur Auswertung des Timer Interrupts
'------------------------------------------------------------------------------
Timer_irq:
   Incr Timercounter                                        ' Timercounter erhhen
   Toggle Led7                                              ' Status LED toggeln
   Timer1 = Timervorgabe                                    ' Timer neu laden

   #if Timer_testmodus
      Print Timercounter
   #endif

   #if Dimmer_testmodus
      Print Sinuscounter
   #endif

Return
'-- End Timer_irq -------------------------------------------------------------


'------------------------------------------------------------------------------
' Interrupt-Service-Routine (INT5): Dimmer_null_irq
' Routine zur Nulldurchgangserkennung des Sinussignals
'------------------------------------------------------------------------------
Dimmer_null_irq:
   ' In dieser IRQ-Routine wird wegen Geschwindigkeit auf Ausgaben verzichtet
   If Free_running_flag = Aktiv Then

   #if Dimmer_testmodus
      Incr Sinuscounter
   #endif

      Start Timer2
      Timer2 = Triac_zuenszeitpunkte(dim_helligkeitsstufe)
   End If

Return
'-- End Dimmer_null_irq -------------------------------------------------------


'------------------------------------------------------------------------------
' Interrupt-Service-Routine (Timer0): Dimmer_triac_irq
' Routine zum Znden des TRIAC
'------------------------------------------------------------------------------
Dimmer_triac_irq:
   Stop Timer2
   Triac_start = 1
   Waitus 20
   Triac_start = 0
Return
'-- End Dimmer_triac_irq ------------------------------------------------------


'******************************************************************************
' Subroutinen
'******************************************************************************


'******************************************************************************
' GOSubroutinen
'******************************************************************************

'------------------------------------------------------------------------------
' REL - Gosub-Routine: Relais_konfig
' Subroutine zur Konfiguration der Relais
'------------------------------------------------------------------------------
Relais_konfig:

   '----- Konfiguration Relais [AUF/ZU/Licht] (Transistorstufe) -----
   'Port G.0 (Licht) als Ausgang definieren und Relais ausschalten
   Config Out_licht = Output
   Rel_licht_on = 0

   'Port C.6 (AUF) als Ausgang definieren und Relais ausschalten
   Config Out_auf = Output
   Rel_auf_on = 0

   'Port C.7 (ZU) als Ausgang definieren und Relais ausschalten
   Config Out_zu = Output
   Rel_zu_on = 0

Return
'-- End Relais_konfig ---------------------------------------------------------


'------------------------------------------------------------------------------
' REL - Gosub-Routine: Relais_auf_an
' Subroutine zum Eimschalten des Relais AUF
'------------------------------------------------------------------------------
Relais_auf_an:

   #if Relais_testmodus
      Print "Enter Relais_auf_an"
   #endif

   Rel_auf_on = 1                                           ' Relais AUF einschalten
Return
'-- End Relais_auf_an ---------------------------------------------------------


'------------------------------------------------------------------------------
' REL - Gosub-Routine: Relais_auf_aus
' Subroutine zum Ausschalten des Relais AUF
'------------------------------------------------------------------------------
Relais_auf_aus:

   #if Relais_testmodus
      Print "Enter Relais_auf_aus"
   #endif

   Rel_auf_on = 0                                           ' Relais AUF ausschalten
Return
'-- End Relais_auf_aus --------------------------------------------------------


'------------------------------------------------------------------------------
' REL - Gosub-Routine: Relais_zu_an
' Subroutine zum Eimschalten des Relais ZU
'------------------------------------------------------------------------------
Relais_zu_an:

   #if Relais_testmodus
      Print "Enter Relais_zu_an"                            ' Relais ZU einschalten
   #endif

   Rel_zu_on = 1
Return
'-- End Relais_zu_an ---------------------------------------------------------


'------------------------------------------------------------------------------
' REL - Gosub-Routine: Relais_zu_aus
' Subroutine zum Ausschalten des Relais ZU
'------------------------------------------------------------------------------
Relais_zu_aus:

   #if Relais_testmodus
      Print "Enter Relais_zu_aus"
   #endif

   Rel_zu_on = 0                                            ' Relais ZU ausschalten
Return
'-- End Relais_zu_aus --------------------------------------------------------


'------------------------------------------------------------------------------
' REL - Gosub-Routine: Relais_licht_an
' Subroutine zum Eimschalten des Relais LICHT
'------------------------------------------------------------------------------
Relais_licht_an:

   #if Relais_testmodus
      Print "Enter Relais_licht_an"
   #endif

   Rel_licht_on = 1                                         ' Relais LICHT einschalten
Return
'-- End Relais_licht_an -------------------------------------------------------


'------------------------------------------------------------------------------
' REL - Gosub-Routine: Relais_licht_aus
' Subroutine zum Ausschalten des Relais LICHT
'------------------------------------------------------------------------------
Relais_licht_aus:

   #if Relais_testmodus
      Print "Enter Relais_licht_aus"
   #endif

   Rel_licht_on = 0                                         ' Relais LICHT ausschalten
Return
'-- End Relais_licht_aus ------------------------------------------------------


'------------------------------------------------------------------------------
' REL - Gosub-Routine: Rel_beschattung_auf
' Subroutine zum kompletten Auffahren der Beschattung mit Richtungsumschaltung
'------------------------------------------------------------------------------
Relais_beschattung_auf:

   #if Relais_testmodus
      Print "Enter Relais_beschattung_auf"
   #endif

   ' Prfung ob Antrieb sich aktuell in ZU-Richtung bewegt
   If Beschattung_zu_flag = Aktiv Then
      ' Wenn JA Antrieb abschalten und Flag zurcksetzen
      Beschattung_zu_flag = Passiv
      Gosub Relais_zu_aus
      Waitms 500                                            ' Zum Schutz des Antriebs 500 ms warten
   End If

   ' Antriebe AUF einschalten und Timerberwachung aktivieren
   Gosub Relais_auf_an
   Beschattung_auf_flag = Aktiv
   Beschattung_auf_counter = Timercounter + Beschattung_delay
Return
'-- End Relais_beschattung_auf ------------------------------------------------


'------------------------------------------------------------------------------
' REL - Gosub-Routine: Relais_beschattung_zu
' Subroutine zum kompletten Zufahren der Beschattung mit Richtungsumschaltung
'------------------------------------------------------------------------------
Relais_beschattung_zu:

   #if Relais_testmodus
      Print "Enter Relais_beschattung_zu"
   #endif

   ' Prfung ob Antrieb sich aktuell in AUF-Richtung bewegt
   If Beschattung_auf_flag = Aktiv Then
      ' Wenn JA Antrieb abschalten und Flag zurcksetzen
      Beschattung_auf_flag = Passiv
      Gosub Relais_auf_aus
      Waitms 500                                            ' Zum Schutz des Antriebs 500 ms warten
   End If

   ' Antriebe ZU einschalten und Timerberwachung aktivieren
   Gosub Relais_zu_an
   Beschattung_zu_flag = Aktiv
   Beschattung_zu_counter = Timercounter + Beschattung_delay
Return
'-- End Relais_beschattung_zu -------------------------------------------------


'------------------------------------------------------------------------------
' REL - Gosub-Routine: Relais_beschattung_stopp
' Subroutine zum Anhalten des aktuell Fahrvorgangs
'------------------------------------------------------------------------------
Relais_beschattung_stop:

   #if Relais_testmodus
      Print "Enter Relais_beschattung_stop"
   #endif

   ' Antrieb ZU anhalten und Timerberwachung zurcksetzen
   Beschattung_zu_flag = Passiv
   Gosub Relais_zu_aus

   ' Antrieb AUF anhalten und Timerberwachung zurcksetzen
   Beschattung_auf_flag = Passiv
   Gosub Relais_auf_aus

Return
'-- End Relais_beschattung_stopp ----------------------------------------------


'------------------------------------------------------------------------------
' Gosub-Routine: Beleuchtung_ein_aus
' Subroutine schaltet Beleuchtung ein oder aus (Toggelbetrieb)
'------------------------------------------------------------------------------
Beleuchtung_ein_aus:

   #if Relais_testmodus
      Print "Enter Beleuchtung_ein_aus"
   #endif

   ' Prfung ob Beleuchtung aus
   If Beleuchtung_flag = Passiv Then
      ' JA, Beleuchtung ist aus und wird eingeschaltet
      Beleuchtung_flag = Aktiv
      Gosub Relais_licht_an
   Else
      ' NEIN, Beleuchtung ist ein und wird ausgeschaltet
      Beleuchtung_flag = Passiv
      Gosub Relais_licht_aus
   End If
Return
'-- End Beleuchtung_ein_aus ---------------------------------------------------


'------------------------------------------------------------------------------
' Gosub-Routine: Sw_timer_sequencer
' Subroutine Abprfen der in RUNNING befindlichen SW-Timer fr Beschattung auf
' oder zu
'------------------------------------------------------------------------------
Sw_timer_sequencer:

   ' berprfung ob Timerberwachung fr Beschattung ZU aktiv
   If Beschattung_zu_flag = Aktiv Then

      ' Prfung ob Timer abgelaufen
      If Timercounter = Beschattung_zu_counter Then

         #if Sequencer_testmodus
            Print "Automatische Abschaltung ZU"
         #endif

         ' Wenn ja Beschattung ZU abschalten und Timerberwachung deaktivieren
         Beschattung_zu_flag = Passiv
         Gosub Relais_zu_aus
      End If
   End If

   ' berprfung ob Timerberwachung fr Beschattung AUF aktiv
   If Beschattung_auf_flag = Aktiv Then

      ' Prfung ob Timer abgelaufen
      If Timercounter = Beschattung_auf_counter Then

         #if Sequencer_testmodus
            Print "Automatische Abschaltung AUF"
         #endif

         ' Wenn ja Beschattung AUF abschalten und Timerberwachung deaktivieren
         Beschattung_auf_flag = Passiv
         Gosub Relais_auf_aus
      End If
   End If

Return
'-- End Sw_timer_sequencer ----------------------------------------------------


'------------------------------------------------------------------------------
' DIM - Gosub-Routine: Dimmer_konfig
' Subroutine zur Konfiguration des Dimemrs
'------------------------------------------------------------------------------
Dimmer_konfig:

   #if Dimmer_testmodus
      Print "Enter Dimmer_konfig"
   #endif

   ' PortE.5 als Eingang mit aktivem PullUp konfigurieren
   Config Null_detect = Input
   Null_detect_pin = 1

   ' PortD.4 als Ausgang definieren und Triac ausschalten
   Config Ignition = Output
   Triac_start = 0


   ' Der Interrupt selbst kann nicht in der Gosub-Routine initialisiert werden
   ' sondern muss im Rumpf des Gesamtprogramms initialisiert werden.

   ' Dimmerhelligkeit auf Default einstellen
   Dim_helligkeitsstufe = Dim_helligkeit_default

   ' Triac Zndzeitpunkte aus Tabelle in Datenstruktur laden
   Restore Triac_zuendtabelle
   For Dim_index_counter = 1 To Dim_helligkeitstabellenwerte
      Read Triac_zuenszeitpunkte(dim_index_counter)
      #if Dimmer_testmodus
         Print "Zuendwert " ; Dim_index_counter ; " : " ; Triac_zuenszeitpunkte(dim_index_counter)
      #endif
   Next Dim_index_counter

Return
'-- End Dimmer_konfig ---------------------------------------------------------


'------------------------------------------------------------------------------
' DIM - Gosub-Routine: Dimmer_heller
' Subroutine zum Erhhen der Dimmerhelligkeit, Verschiebung Index Zndzeitpunkt
'------------------------------------------------------------------------------
Dimmer_heller:

   #if Dimmer_testmodus
      Print "Enter Dimmer_heller"
   #endif

   If Dim_helligkeitsstufe < Dim_helligkeit_max Then
      Incr Dim_helligkeitsstufe
   End If

   #if Dimmer_testmodus
      Print "Helligkeit ist : " ; Dim_helligkeitsstufe
   #endif

Return
'-- End Dimmer_heller ---------------------------------------------------------


'------------------------------------------------------------------------------
' DIM - Gosub-Routine: Dimmer_dunkler
' Subroutine zum reduzieren der Dimmerhelligkeit, Verschiebung Index
' Zndzeitpunkt
'------------------------------------------------------------------------------
Dimmer_dunkler:

   #if Dimmer_testmodus
      Print "Enter Dimmer_dunkler"
   #endif

   If Dim_helligkeitsstufe > Dim_helligkeit_min Then
      Decr Dim_helligkeitsstufe
   End If

   #if Dimmer_testmodus
      Print "Helligkeit ist : " ; Dim_helligkeitsstufe
   #endif

Return
'-- End Dimmer_dunkler --------------------------------------------------------


'------------------------------------------------------------------------------
' Gosub-Routine: Dimmer_free_running
' Subroutine zum Setzen von Parametern fr freien Dimmerlauf, gesteuert ber
' Nulldurchgangskennung
'------------------------------------------------------------------------------
Dimmer_free_running:

   #if Dimmer_testmodus
      Print "Enter Dimmer_free_running"
   #endif

   ' Prfung ob free_running aktiv
   If Free_running_flag = Passiv Then
      ' JA, Free_runing ist aus und wird eingeschaltet
      Free_running_flag = Aktiv

   #if Dimmer_testmodus
      Print "Free Running ist aktiv"
   #endif

      Led6 = Led_an
   Else
      ' NEIN, Free_running ist ein und wird ausgeschaltet
      Free_running_flag = Passiv

   #if Dimmer_testmodus
      Print "Free Running ist passiv"
   #endif

      Led6 = Led_aus
   End If


Return
'-- End Dimmer_free_running ---------------------------------------------------


'------------------------------------------------------------------------------
' Gosub-Routine: Dimmer_one_shot
' Subroutine zum einmaligen Auslsen des Timer0 fr TRIAC-Zndung
'------------------------------------------------------------------------------
Dimmer_one_shot:

   #if Dimmer_testmodus
      Print "Enter Dimmer_one_shot"
   #endif

   Led0 = Led_an
   Waitus 5
   Led0 = Led_aus
   Start Timer2
   Timer2 = Triac_zuenszeitpunkte(dim_helligkeitsstufe)


Return
'-- End Dimmer_one_shot -------------------------------------------------------


End                                                         'end program


'------------------------------------------------------------------------------
' Definition von globalen Konstantenfeldern
'------------------------------------------------------------------------------

Triac_zuendtabelle:
' Tabelle mit Originalwerten
' Data 156 , 139 , 131 , 125 , 120 , 116 , 111 , 108
' Data 103 , 100 , 95 , 92 , 89 , 86 , 83 , 80
' Data 77 , 73 , 70 , 67 , 64 , 61 , 56 , 53,
' Data 50 , 45 , 41 , 36 , 31 , 25 , 17 , 0
' Fr Timer0 werden Einerkompliment-Werte (255-Wert) bentigt
Data 99 , 116 , 124 , 130 , 135 , 139 , 144 , 147
Data 152 , 155 , 160 , 163 , 166 , 169 , 172 , 175
Data 178 , 182 , 185 , 188 , 191 , 194 , 199 , 202
Data 205 , 210 , 214 , 219 , 224 , 230 , 238 , 250


'###################################### END ###################################